
#ifndef _herbe_
#define _herbe_

//include io
#include <stdio.h>
#include <stdlib.h>

//include GL
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

//include base prairie
#include "GrassDefines.h"
#include "GrassRandom.h"
#include "SharedData.h"
#include "GeometrieBase.hpp"
#include "InformationPrimitive.h"
#include "VolumeVision.hpp"

/** \class Herbe
\brief regroupe les informations necessaire a l'affichage d'une herbe

pos0, pos1, tan0, tan1 permettent de construire une herbe de la forme
souhaitee grace aux courbes de Hermite

*/

class Herbe
{
public:
	///epaisseur moy des herbes
	static GReal ms_height;
	static GReal ms_width;
	
	static GReal m_noisePositions;
	static GReal m_noiseNormals;
public:
	///position racine
	Point3 _pos0;
	///position extremite
	Point3 _pos1;
	///tangeante racine
	Vec3 _tan0;
	///tangeante extremite
	Vec3 _tan1;	
	///normale racine
	GReal _norm0z;
	///normale extremite
	GReal _norm1z;
public:	
	///constructeur par defaut : les variables ne sont pas initialisees
	Herbe() {}
	
	///constructeur par recopie
	Herbe(const Herbe& h);
	
	///surdefinition de l'affectation
	inline Herbe& operator=(const Herbe& h);

	///destructeur  
	~Herbe() {}	
	
	///affichage specifique (utilise pour creer la texture 2D5)
	//La seule difference (pour l'instant) est de pouvoir 
	//preciser un coefficient d'epaisseur
	
	
	///affichage de l'herbe
	inline void Draw(const short& NbPolygones) const;
	void Draw (const short& NbPolygones, const Vec3& orientation, const GReal& coefWidth) const;
   
	///calcul uniquement la valeur des normales
	///de l'herbes suivant l'information de vent et d'ecrasement.
	inline void MakeNormals(const InformationVent& informationVent,
				const GReal& crushingPower);
	///calcul entierement la forme de l'herbe (sauf position) 
	///suivant l'information de vent et d'ecrasement
	inline void Make(const InformationVent& informationVent,
			 const Vec2&  crushingDirection,
			 const GReal& crushingPower,
			 const GReal& hauteur);
	inline void MakeWithWind(const InformationVent& informationVent,
			   	 const GReal& hauteur);
	//calcul la forme de l'herbe (sauf position) en prenant en compte sa forme 
	//courante et en ajoutant l'information d'ecrasement
	inline void ModifyWithCrushing(const GReal windPower,
			               const Vec2&  crushingDirection,
		 	   	       const GReal& crushingPower,
			   	       const GReal& hauteur);
	///calcul entierement la forme de l'herbe (+ position) 
	///par interpolation lineaire
	inline void Make(const Herbe& grass1,
			 const Herbe& grass2,
			 const GReal transition);
	///calcul entierement la forme de l'herbe (avec position=position de grass1) 
	///par interpolation lineaire
	inline void MakeAtFirstGrassPosition(const Herbe& grass1,
			 		     const Herbe& grass2,
			 		     const GReal transition);
	inline void MakeAtFirstGrassPosition(const Herbe& grass1,
					     const GReal power1,
			 		     const Herbe& grass21,
					     const GReal power21,
					     const Herbe& grass22,
					     const GReal power22
			 		     );
	///calcul entierement la forme de l'herbe (+ position) 
	///par interpolation bilineaire
	inline void Make(const Herbe& grass00,
			 const Herbe& grass10,
			 const Herbe& grass01,
			 const Herbe& grass11,
			 const GReal& transition10,
			 const GReal& transition01);
	///calcul entierement la forme de l'herbe (+ position) 
	///par interpolation bilineaire + bruit
	inline void MakeWithNoise(const Herbe& grass00,
			   	  const Herbe& grass10,
			   	  const Herbe& grass01,
			   	  const Herbe& grass11,
			   	  const GReal& transition10,
			   	  const GReal& transition01);		   
};

void Herbe::Draw(const short& NbPolygones) const
{	
static Vec3 billboard(0.0f,0.0f,0.0f);
static Vec3 axe(0.0f,0.0f,0.0f);
static Vec3 axeBillboard;
GReal x, y, z, pas, compteur_pas;
GReal demims_width_max,demims_width_min,demims_width;
GReal type_texture=GRand();

//initialisation compteur
compteur_pas= 0.0f;
pas=1.0f/NbPolygones;

//initialisation epaisseur	
GRand(demims_width_max);
demims_width_max*=0.02f;demims_width_max+=ms_width;
GRand(demims_width_min);
demims_width_min*=0.01f;demims_width=demims_width_max+demims_width_min;

//calcul du billboard			
billboard.x=SharedData::m_cameraBillboard.x*demims_width;
billboard.y=SharedData::m_cameraBillboard.y*demims_width;
billboard.z=0.0;	

//decalageNorm
GReal decalageNorm=0.3f+0.69f*GRand();
GReal norm0z=_norm0z;
GReal norm1z=_norm1z;
GReal norm2z=decalageNorm*_norm1z+(1.0f-decalageNorm)*_norm0z;

//premier calcule -> x,y,z sont connus
x = _pos0.x;y = _pos0.y;z = _pos0.z;

//debut affichage	
glBegin(GL_QUAD_STRIP);
glNormal3f(0.0f,0.0f,_norm0z);
glTexCoord2f(0.0f,type_texture);
glVertex3f(x-billboard.x, y-billboard.y, z);
glTexCoord2f(1.0f,type_texture);
glVertex3f(x+billboard.x, y+billboard.y, z);

for (int i=1;i<=NbPolygones;i++)
	{			 				
	compteur_pas = i*pas;				
	//sauvegarde ancienne valeur
	axe.x=-x;axe.y=-y;axe.z=-z;
	//calcul du nouveau point	
	x = Hermite(compteur_pas,_pos0.x,_tan0.x,_pos1.x,_tan1.x);
	y = Hermite(compteur_pas,_pos0.y,_tan0.y,_pos1.y,_tan1.y);
	z = Hermite(compteur_pas,_pos0.z,_tan0.z,_pos1.z,_tan1.z);	
	//calcul nouvel epaisseur
	demims_width = (1.0f-compteur_pas*compteur_pas)*demims_width_max+demims_width_min; 
	//calcul nouveau vecteur normal -> que a partir de extremite herbe -> vitesse	
	//calcul vecteur axe herbe
	axe.x+=x;axe.y+=y;axe.z+=z;	
	//calcul vecteur billboard
	billboard.x=SharedData::m_cameraBillboard.x*demims_width;
	billboard.y=SharedData::m_cameraBillboard.y*demims_width;
	billboard.z=0.0f;
	//detection simple -> si la courbure est superieur a 45 degrees
	if (Abs(axe.x) > axe.z || Abs(axe.y) >axe.z)
		{
		//modifiaction du billboard pour obtenir un meilleur rendu de la courbure	
		//projection axe herbe sur billboard
		axeBillboard = billboard*(axe*billboard)/(demims_width*demims_width);
		//calcul axe du nouveau vecteur billboard
		if (axe*billboard>0.0f) billboard.z-=Abs(axeBillboard.x+axeBillboard.y);
		else billboard.z+=Abs(axeBillboard.x+axeBillboard.y);	
		//le billboard prend une norme = demi-epaisseur
		billboard.normalize();
		billboard*=demims_width;	
		}
	if (compteur_pas<decalageNorm)
	{
	compteur_pas=compteur_pas/decalageNorm;
	glNormal3f(0.0f,0.0f,(compteur_pas*norm1z+(1.0f-compteur_pas)*norm0z));
	}
	else
	{
	compteur_pas=(compteur_pas-decalageNorm)/(1.0f-decalageNorm);
	glNormal3f(0.0f,0.0f,(compteur_pas*norm2z+(1.0f-compteur_pas)*norm1z));
	}
	glTexCoord2f(0.0f,type_texture);
	glVertex3f(x-billboard.x, y-billboard.y, z-billboard.z);
	
	glTexCoord2f(1.0f,type_texture);
	glVertex3f(x+billboard.x, y+billboard.y, z+billboard.z);
	}
glEnd();
}	
	

//donnees issues du precalcul
#define VENT_TAN0_COURBURE_X 	2.8f
#define VENT_COEF_VARIATION_X 	0.8f
#define VENT_TAN1_COURBURE_X 	0.0f

#define VENT_TAN0_COURBURE_Y	2.8f
#define VENT_COEF_VARIATION_Y	0.8f 
#define VENT_TAN1_COURBURE_Y	0.0f

#define VENT_TAN0_COURBURE_Z	0.0f
#define VENT_COEF_VARIATION_Z	0.5f
#define VENT_TAN1_COURBURE_Z	-0.6f

#define ECRASEMENT_TAN0_COURBURE_X	3.0f
#define ECRASEMENT_COEF_VARIATION_X	0.92f 
#define ECRASEMENT_TAN1_COURBURE_X	0.0f

#define ECRASEMENT_TAN0_COURBURE_Y	3.0f
#define ECRASEMENT_COEF_VARIATION_Y	0.92f 
#define ECRASEMENT_TAN1_COURBURE_Y	0.0f

#define ECRASEMENT_TAN0_COURBURE_Z	0.0f
#define ECRASEMENT_COEF_VARIATION_Z	0.0f
#define ECRASEMENT_TAN1_COURBURE_Z	-1.0f

#define MIN_NORM0_Z			0.3f
#define MAX_AUGMENTATION_NORM0_Z	0.3f

#define MIN_NORM1_Z			0.6f
#define MAX_AUGMENTATION_NORM1_Z	0.4f

Herbe& Herbe::operator=(const Herbe& h)
		{
		_pos0.x=h._pos0.x;
		_pos0.y=h._pos0.y;
		_pos0.z=h._pos0.z;
		_pos1.x=h._pos1.x;
		_pos1.y=h._pos1.y;
		_pos1.z=h._pos1.z;
		
		_tan0.x=h._tan0.x;
		_tan0.y=h._tan0.y;
		_tan0.z=h._tan0.z;
		_tan1.x=h._tan1.x;
		_tan1.y=h._tan1.y;
		_tan1.z=h._tan1.z;
		
		_norm0z=h._norm0z;
		_norm1z=h._norm1z;
		return *this;
		}

void Herbe::MakeNormals(const InformationVent& informationVent,
			    const GReal& crushingPower)
{
GReal normeVent;
normeVent = sqrt(informationVent._vecteur.x*informationVent._vecteur.x
		+informationVent._vecteur.y*informationVent._vecteur.y);
_norm1z = MIN_NORM1_Z+MAX_AUGMENTATION_NORM1_Z*normeVent;
_norm0z = MIN_NORM0_Z;
if (crushingPower!=0.0f)
  {
  GReal norm0;
  GReal norm1;
  norm0 = MIN_NORM0_Z+MAX_AUGMENTATION_NORM0_Z;
  norm1 = MIN_NORM1_Z+MAX_AUGMENTATION_NORM1_Z;
  GReal ratioVent = (informationVent._power)/(crushingPower+informationVent._power);
  GReal ratioEcrasement = 1.0f-ratioVent;
  _norm1z= ratioEcrasement * norm1+ ratioVent * _norm1z;
  _norm0z= ratioEcrasement * norm0+ ratioVent * _norm0z;		
  }	
}


void Herbe::Make(const InformationVent& informationVent,
		 const Vec2&  crushingDirection,
		 const GReal& crushingPower,
		 const GReal& hauteur)
{
GReal varXY;
//MODIFICATION VENT	
GReal normeVent;
normeVent = sqrt(informationVent._vecteur.x*informationVent._vecteur.x
		+informationVent._vecteur.y*informationVent._vecteur.y);		
//calcul de la nouvelle position de l'extremite de l'herbe		
varXY = Hermite(normeVent,
		0.0f,VENT_TAN0_COURBURE_X,
		VENT_COEF_VARIATION_X * hauteur,VENT_TAN1_COURBURE_X); 
if (normeVent==0.0f)
  {
  _pos1.x=_pos0.x;
  _pos1.y=_pos0.y;
  }
else
  {
  _pos1.x = _pos0.x + varXY * informationVent._vecteur.x/normeVent;
  _pos1.y = _pos0.y + varXY * informationVent._vecteur.y/normeVent;	
  }
_pos1.z = Hermite(normeVent,
		  _pos0.z + hauteur,
		  VENT_TAN0_COURBURE_Z,
		  _pos0.z + VENT_COEF_VARIATION_Z * hauteur,
		  VENT_TAN1_COURBURE_Z);
//calcul nouvelle tangeante et norme extremite de l'herbe		
_tan0.x = 0.0f;
_tan0.y = 0.0f;
_tan0.z = hauteur;
_tan1.x = informationVent._vecteur.x;
_tan1.y = informationVent._vecteur.y;
_tan1.z = 1.0f-normeVent;	
_norm1z = MIN_NORM1_Z+MAX_AUGMENTATION_NORM1_Z*normeVent;
_norm0z = MIN_NORM0_Z;
//FIN MODIFICATION VENT

//MODIFICATION ECRASEMENT
if (crushingDirection.x!=0.0f || crushingDirection.y!=0.0f)
    {	
    Vec3 tan0, tan1;
    Point3 pos1;
    GReal norm0, norm1, ratioVent, ratioEcrasement;
    Vec2 vecteurEcrasement(crushingDirection.x,crushingDirection.y);
    vecteurEcrasement.normalize();

    //on prend les valeurs  positives du vecteur de ecrasement 
    //pour pouvoir applique la courbe d'hermite		
    //calcul de la nouvelle position de l'extremite de l'herbe
    varXY = ECRASEMENT_COEF_VARIATION_X * hauteur;	
    pos1.x = _pos0.x + varXY * vecteurEcrasement.x;
    pos1.y = _pos0.y + varXY * vecteurEcrasement.y;
    pos1.z = _pos0.z + ECRASEMENT_COEF_VARIATION_Z * hauteur;
    tan1.x = vecteurEcrasement.x;
    tan1.y = vecteurEcrasement.y;
    tan1.z = -0.5f;
    tan0.x = vecteurEcrasement.x;
    tan0.y = vecteurEcrasement.y;
    tan0.z = 0.8f*hauteur;
    norm0 = MIN_NORM0_Z+MAX_AUGMENTATION_NORM0_Z;
    norm1 = MIN_NORM1_Z+MAX_AUGMENTATION_NORM1_Z;
    
    //interpolation
    ratioVent = (informationVent._power)/(crushingPower+informationVent._power);
    ratioEcrasement = 1.0f-ratioVent;
    _pos1 = ratioEcrasement * pos1 + ratioVent * _pos1;
    _tan0 = ratioEcrasement * tan0 + ratioVent * _tan0;
    _tan1 = ratioEcrasement * tan1 + ratioVent * _tan1;
    _norm1z= ratioEcrasement * norm1+ ratioVent * _norm1z;
    _norm0z= ratioEcrasement * norm0+ ratioVent * _norm0z;		
    }
//FIN MODIFICATION ECRASEMENT
}

void Herbe::MakeWithWind(const InformationVent& informationVent,
		 	 const GReal& hauteur)
{
GReal varXY;
GReal normeVent;
normeVent = sqrt(informationVent._vecteur.x*informationVent._vecteur.x
		+informationVent._vecteur.y*informationVent._vecteur.y);		
//calcul de la nouvelle position de l'extremite de l'herbe		
varXY = Hermite(normeVent,
		0.0f,
		VENT_TAN0_COURBURE_X,
		VENT_COEF_VARIATION_X*hauteur,
		VENT_TAN1_COURBURE_X); 
if (normeVent==0.0f) {_pos1.x=_pos0.x;_pos1.y=_pos0.y;}
else
  {
  _pos1.x = _pos0.x + varXY * informationVent._vecteur.x/normeVent;
  _pos1.y = _pos0.y + varXY * informationVent._vecteur.y/normeVent;	
  }
_pos1.z = Hermite(normeVent,
		  _pos0.z + hauteur,
		  VENT_TAN0_COURBURE_Z,
		  _pos0.z + VENT_COEF_VARIATION_Z * hauteur,
		  VENT_TAN1_COURBURE_Z);
//calcul nouvelle tangeante et norme extremite de l'herbe		
_tan0.x = 0.0f;
_tan0.y = 0.0f;
_tan0.z = hauteur;
_tan1.x = informationVent._vecteur.x;
_tan1.y = informationVent._vecteur.y;
_tan1.z = 1.0f-normeVent;	
_norm1z = MIN_NORM1_Z+MAX_AUGMENTATION_NORM1_Z*normeVent;
_norm0z = MIN_NORM0_Z;
}

void Herbe::ModifyWithCrushing(const GReal windPower,
			       const Vec2&  crushingDirection,
		 	       const GReal& crushingPower,
			       const GReal& hauteur)
{
if (crushingDirection.x!=0.0f || crushingDirection.y!=0.0f)
    {	
    Vec3 tan0, tan1;
    Point3 pos1;
    GReal varXY, norm0, norm1, ratioVent, ratioEcrasement;
    Vec2 vecteurEcrasement(crushingDirection.x,crushingDirection.y);
    vecteurEcrasement.normalize();

    //on prend les valeurs  positives du vecteur de ecrasement 
    //pour pouvoir applique la courbe d'hermite		
    //calcul de la nouvelle position de l'extremite de l'herbe
    varXY = ECRASEMENT_COEF_VARIATION_X * hauteur;	
    pos1.x = _pos0.x + varXY * vecteurEcrasement.x;
    pos1.y = _pos0.y + varXY * vecteurEcrasement.y;
    pos1.z = _pos0.z + ECRASEMENT_COEF_VARIATION_Z * hauteur;
    tan1.x = vecteurEcrasement.x;
    tan1.y = vecteurEcrasement.y;
    tan1.z = -0.5f;
    tan0.x = vecteurEcrasement.x;
    tan0.y = vecteurEcrasement.y;
    tan0.z = 0.8f*hauteur;
    norm0 = MIN_NORM0_Z+MAX_AUGMENTATION_NORM0_Z;
    norm1 = MIN_NORM1_Z+MAX_AUGMENTATION_NORM1_Z;
    
    //interpolation
    ratioVent = (windPower)/(crushingPower+windPower);
    ratioEcrasement = 1.0f-ratioVent;
    _pos1 = ratioEcrasement * pos1 + ratioVent * _pos1;
    _tan0 = ratioEcrasement * tan0 + ratioVent * _tan0;
    _tan1 = ratioEcrasement * tan1 + ratioVent * _tan1;
    _norm1z= ratioEcrasement * norm1+ ratioVent * _norm1z;
    _norm0z= ratioEcrasement * norm0+ ratioVent * _norm0z;		
    }
}


void Herbe::Make(const Herbe& grass1, const Herbe& grass2, const GReal transition)
{
GReal _transition = 1.0f-transition;

_pos0.x=_transition* grass1._pos0.x + transition* grass2._pos0.x;
_pos0.y=_transition* grass1._pos0.y + transition* grass2._pos0.y;
_pos0.z=_transition* grass1._pos0.z + transition* grass2._pos0.z;
	   
_pos1.x=_transition* grass1._pos1.x + transition* grass2._pos1.x;
_pos1.y=_transition* grass1._pos1.y + transition* grass2._pos1.y;	
_pos1.z=_transition* grass1._pos1.z + transition* grass2._pos1.z;
		
_tan0.x=_transition* grass1._tan0.x + transition* grass2._tan0.x;
_tan0.y=_transition* grass1._tan0.y + transition* grass2._tan0.y;
_tan0.z=_transition* grass1._tan0.z + transition* grass2._tan0.z;

_tan1.x=_transition* grass1._tan1.x + transition* grass2._tan1.x;	
_tan1.y=_transition* grass1._tan1.y + transition* grass2._tan1.y;	
_tan1.z=_transition* grass1._tan1.z + transition* grass2._tan1.z;

_norm0z=_transition* grass1._norm0z + transition* grass2._norm0z;
_norm1z=_transition* grass1._norm1z + transition* grass2._norm1z;
}

void Herbe::MakeAtFirstGrassPosition(const Herbe& grass1, const Herbe& grass2, const GReal transition)
{
GReal _transition = 1.0f-transition;

_pos0.x = grass1._pos0.x;
_pos0.y = grass1._pos0.y;
_pos0.z = grass1._pos0.z;

_tan0.x=_transition* grass1._tan0.x + transition* grass2._tan0.x;
_tan0.y=_transition* grass1._tan0.y + transition* grass2._tan0.y;
_tan0.z=_transition* grass1._tan0.z + transition* grass2._tan0.z;

_tan1.x=_transition* grass1._tan1.x + transition* grass2._tan1.x;	
_tan1.y=_transition* grass1._tan1.y + transition* grass2._tan1.y;	
_tan1.z=_transition* grass1._tan1.z + transition* grass2._tan1.z;

_pos1.x=_transition*(grass1._pos1.x-grass1._pos0.x)
        +transition*(grass2._pos1.x-grass2._pos0.x)
	+_pos0.x;
_pos1.y=_transition*(grass1._pos1.y-grass1._pos0.y)
        +transition*(grass2._pos1.y-grass2._pos0.y)
	+_pos0.y;
_pos1.z=_transition*(grass1._pos1.z-grass1._pos0.z)
        +transition*(grass2._pos1.z-grass2._pos0.z)
	+_pos0.z;
}

void Herbe::MakeAtFirstGrassPosition(const Herbe& grass1,
				     const GReal power1,
			 	     const Herbe& grass21,
				     const GReal power21,
				     const Herbe& grass22,
				     const GReal power22)
{
_pos0.x=grass1._pos0.x;
_pos0.y=grass1._pos0.y;
_pos0.z=grass1._pos0.z;

_tan0.x=power1*grass1._tan0.x+ power21* grass21._tan0.x + power22*grass22._tan0.x;
_tan0.y=power1*grass1._tan0.y+ power21* grass21._tan0.y + power22*grass22._tan0.y;
_tan0.z=power1*grass1._tan0.z+ power21* grass21._tan0.z + power22*grass22._tan0.z;

_tan1.x=power1* grass1._tan1.x + power21* grass21._tan1.x + power22*grass22._tan1.x;
_tan1.y=power1* grass1._tan1.y + power21* grass21._tan1.y + power22*grass22._tan1.y;
_tan1.z=power1* grass1._tan1.z + power21* grass21._tan1.z + power22*grass22._tan1.z;

_pos1.x= power1 *(grass1._pos1.x-grass1._pos0.x)
        +power21*(grass21._pos1.x-grass21._pos0.x)
	+power22*(grass22._pos1.x-grass22._pos0.x)
	+_pos0.x;
_pos1.y= power1 *(grass1._pos1.y-grass1._pos0.y)
        +power21*(grass21._pos1.y-grass21._pos0.y)
	+power22*(grass22._pos1.y-grass22._pos0.y)
	+_pos0.y;
_pos1.z= power1 *(grass1._pos1.z-grass1._pos0.z)
        +power21*(grass21._pos1.z-grass21._pos0.z)
	+power22*(grass22._pos1.z-grass22._pos0.z)
	+_pos0.z;
}


void Herbe::Make(const Herbe& grass00, const Herbe& grass10,
		 const Herbe& grass01, const Herbe& grass11,
		 const GReal& transition10, const GReal& transition01)
{
GReal coef00,coef01,coef11,coef10;
coef00= (1.0f-transition01) * (1.0f-transition10);
coef01= transition01 * (1.0f-transition10);
coef11= transition01 * transition10;
coef10= (1.0f-transition01) * transition10;

_pos0.x= coef00 * grass00._pos0.x + coef10 * grass10._pos0.x
       + coef01 * grass01._pos0.x + coef11 * grass11._pos0.x;
_pos0.y= coef00 * grass00._pos0.y + coef10 * grass10._pos0.y
       + coef01 * grass01._pos0.y + coef11 * grass11._pos0.y;
_pos0.z= coef00 * grass00._pos0.z + coef10 * grass10._pos0.z
       + coef01 * grass01._pos0.z + coef11 * grass11._pos0.z;

_pos1.x= coef00 * grass00._pos1.x + coef10 * grass10._pos1.x
       + coef01 * grass01._pos1.x + coef11 * grass11._pos1.x;
_pos1.y= coef00 * grass00._pos1.y + coef10 * grass10._pos1.y
       + coef01 * grass01._pos1.y + coef11 * grass11._pos1.y;
_pos1.z= coef00 * grass00._pos1.z + coef10 * grass10._pos1.z
       + coef01 * grass01._pos1.z + coef11 * grass11._pos1.z;

_tan0.x= coef00 * grass00._tan0.x + coef10 * grass10._tan0.x
       + coef01 * grass01._tan0.x + coef11 * grass11._tan0.x;
_tan0.y= coef00 * grass00._tan0.y + coef10 * grass10._tan0.y
       + coef01 * grass01._tan0.y + coef11 * grass11._tan0.y;
_tan0.z= coef00 * grass00._tan0.z + coef10 * grass10._tan0.z
       + coef01 * grass01._tan0.z + coef11 * grass11._tan0.z;

_tan1.x= coef00 * grass00._tan1.x + coef10 * grass10._tan1.x
       + coef01 * grass01._tan1.x + coef11 * grass11._tan1.x;
_tan1.y= coef00 * grass00._tan1.y + coef10 * grass10._tan1.y
       + coef01 * grass01._tan1.y + coef11 * grass11._tan1.y;
_tan1.z= coef00 * grass00._tan1.z + coef10 * grass10._tan1.z
       + coef01 * grass01._tan1.z + coef11 * grass11._tan1.z;

_norm0z= coef00 * grass00._norm0z + coef10 * grass10._norm0z
       + coef01 * grass01._norm0z + coef11 * grass11._norm0z;
_norm1z= coef00 * grass00._norm1z + coef10 * grass10._norm1z
       + coef01 * grass01._norm1z + coef11 * grass11._norm1z;
}

void Herbe::MakeWithNoise(const Herbe& grass00, const Herbe& grass10,
			  const Herbe& grass01, const Herbe& grass11,
			  const GReal& transition10, const GReal& transition01)
{
GReal coef00,coef01,coef11,coef10;
coef00= (1.0f-transition01) * (1.0f-transition10);
coef01= transition01 * (1.0f-transition10);
coef11= transition01 * transition10;
coef10= (1.0f-transition01) * transition10;

_pos0.x= coef00 * grass00._pos0.x + coef10 * grass10._pos0.x
       + coef01 * grass01._pos0.x + coef11 * grass11._pos0.x;
_pos0.y= coef00 * grass00._pos0.y + coef10 * grass10._pos0.y
       + coef01 * grass01._pos0.y + coef11 * grass11._pos0.y;
_pos0.z= coef00 * grass00._pos0.z + coef10 * grass10._pos0.z
       + coef01 * grass01._pos0.z + coef11 * grass11._pos0.z;

_pos1.x= coef00 * grass00._pos1.x + coef10 * grass10._pos1.x
       + coef01 * grass01._pos1.x + coef11 * grass11._pos1.x;
_pos1.y= coef00 * grass00._pos1.y + coef10 * grass10._pos1.y
       + coef01 * grass01._pos1.y + coef11 * grass11._pos1.y;
_pos1.z= coef00 * grass00._pos1.z + coef10 * grass10._pos1.z
       + coef01 * grass01._pos1.z + coef11 * grass11._pos1.z;

_tan0.x= coef00 * grass00._tan0.x + coef10 * grass10._tan0.x
       + coef01 * grass01._tan0.x + coef11 * grass11._tan0.x;
_tan0.y= coef00 * grass00._tan0.y + coef10 * grass10._tan0.y
       + coef01 * grass01._tan0.y + coef11 * grass11._tan0.y;
_tan0.z= coef00 * grass00._tan0.z + coef10 * grass10._tan0.z
       + coef01 * grass01._tan0.z + coef11 * grass11._tan0.z;

_tan1.x= coef00 * grass00._tan1.x + coef10 * grass10._tan1.x
       + coef01 * grass01._tan1.x + coef11 * grass11._tan1.x;
_tan1.y= coef00 * grass00._tan1.y + coef10 * grass10._tan1.y
       + coef01 * grass01._tan1.y + coef11 * grass11._tan1.y;
_tan1.z= coef00 * grass00._tan1.z + coef10 * grass10._tan1.z
       + coef01 * grass01._tan1.z + coef11 * grass11._tan1.z;

_norm0z= coef00 * grass00._norm0z + coef10 * grass10._norm0z
       + coef01 * grass01._norm0z + coef11 * grass11._norm0z;
_norm1z= coef00 * grass00._norm1z + coef10 * grass10._norm1z
       + coef01 * grass01._norm1z + coef11 * grass11._norm1z;
              
//bruit
if (m_noisePositions>0.0)
   {
   GReal random[3];
   GRand(random[0]);random[0]-=0.5f;random[0]*=m_noisePositions;
   GRand(random[1]);random[1]-=0.5f;random[1]*=m_noisePositions;
   GRand(random[2]);random[2]-=0.5f;random[2]*=m_noisePositions;
   _pos1.x+=random[0];_tan1.x+=random[0];
   _pos1.y+=random[1];_tan1.y+=random[1];
   _pos1.z+=random[2];_tan1.z+=random[2];
   }
//_norm1z=Min(_norm1z+m_noiseNormals*(GRand()-0.5f),1.0f);
}			 


#endif /* _herbe_ */
