
#ifndef _noeud_
#define _noeud_

//include io
#include <stdio.h>
#include <stdlib.h>

//include GL
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>

//include base prairie
#include "GrassDefines.h"
#include "SharedData.h"
#include "GrassInterface.h"
#include "GeometrieBase.hpp"
#include "InformationPrimitive.h"

//autres
#include "VolumeVision.hpp"
#include "Herbe.hpp"



/** \class Noeud
\brief echantillonnage du terrain 

rassemble un ensemble d'informations en un point precis du terrain
-forme de l'herbe 
-vecteur normal au terrain
-distance au marcheur
-type d'affichage
-densite de l'herbe
-visibilite ou non

hormis la position, la _hauteur et le vecteur normal au terrain, les informations sont remis a jour avant chaque affichage
*/
class Noeud
{
public:

	///Vrai si le noeud est actif
	bool m_isActive;
	/// Vrai si le Noeud est visible
	bool m_isVisible;
	/// Vrai si les informations du Noeud doivent etre mise a jour
	bool m_mustBeUpdated;
	
	///contient les donnees de position et de courbure 
	Herbe m_grass;
	///donnee constante : position du noeud
	Point3 m_position;	
	///donnee constante :vecteur de courbure au repos
	/// la norme de ce vecteur ne doit pas de passer 1
	Vec2 m_initGrassCurve;
	///donnee constante : hauteur de l'herbe en ce noeud
	
	/// hauteur reel
	GReal m_heightGrass;	
	/// hauteur Modifie (pour transition (3D/2D5 -> 2D))
	GReal m_heightGrassDraw;
	//hauteur des points de texture aux noeuds
	GReal m_heightTex2D;
	
	///donnee reel :densite de base autour de ce noeud  [0.0;1.0]
	GReal m_grassDensity;
	/// densite modifie (eventuellement suivant ditance)
	GReal m_grassDensityDraw;
	
	/// distance du noeud par rapport au marcheur
	GReal m_distanceToCamera;
	// vecteur normal au terrain
	Vec3 m_groundNormal;
	
	/// InformationVent en ce noeud
	InformationVent m_windInformation;
	
	/// type d affichage deduit pour ce noeud suivant la distance
	short m_typeOfDrawing;
	
public:
	Noeud();
	
	/// destructeur	 
	~Noeud() {}
	
	//---------------------------------------------------------------------
	//---------------------------ATTRIBUTE---------------------------------
	//---------------------------------------------------------------------
	
	///position
	void setPosition(const Point3& position);
	
	///hauteur
	void SetGrassHeight(const GReal& hauteur);
	void setTex2DHeight(GReal heigth);

	//---------------------------------------------------------------------
	//---------------------------PRE DRAW----------------------------------
	//---------------------------------------------------------------------
	
	/// calcul si le noeud est visible ou non
	/// et met a jour la variable _visible
	inline void UpdateVisibilite(const GReal& imprecision);	
	
	
	/// calcul le type d'affichqge de l'herbe suivant sa distance au marcheur
	inline void UpdateTypeAffichage();
	/// calcul densite d'herbe en ce noeud
	inline void UpdateDensite();
	///mise a jour de l'influence des primitives de vent
	inline void UpdateInfluencePrimitives();
	/// fabrication de l'herbe en ce noeud a partir des informations de position, 
	///de vent, de hauteur...
	inline void FabricationHerbe();	
	
	
	/// UpdatePreDraw() == UpdateTypeAffichage() + UpdateDensite() 
	///         + UpdateInfluencePrimitives() + FabricationHerbe()
	inline void UpdatePreDraw();
	
};


class NodeCrushing
{
public:
	static GReal ms_crushingMaxTime;
	static GReal ms_crushingDamping;

	///Vrai si le noeud est actif
	bool m_isActive;
	bool m_isCrushed;
	
	//position
	Point3 m_position;		
	///contient les donnees de position et de courbure 
	Herbe m_grass;
	
	/// information en rapport a l'ecrasement
	Vec2 m_crushingDirection;
	GReal m_crushingMaxTime;
	GReal m_crushingCurrentTime;
	GReal m_crushingMaxPower;
	GReal m_crushingCurrentPower;
	
public:
	NodeCrushing();
	
	///position
	void setPosition(const Point3& position);
	
	/// destructeur	 
	~NodeCrushing() {}
	
	///initialisation des donnees du noeud
	/// ; necessaire avant chaque mise a jour des informations
	inline void UpdateCrushing(const GReal& dt);		
	///mise a jour de l'influence d'un element de trajectoire sur ce noeud
	inline GReal UpdateInfluenceTrajectoire(const Point3& debutSegmentTrajectoire,
       					const Point3& finSegmentTrajectoire,
					const Vec2& vDirectionTrajectoire,
					const Vec2& vNormalTrajectoire,
					const GReal& powerCrushing,
					const GReal& coefCrushingAccordingToNormal);
	
	/// fabrique la forme d'une herbe suivant 
	/// les informations des voisins			  	
	inline void Make (const Noeud* n);
	inline void Make (const Noeud* n0,const Noeud* n1);
	inline void Make (const Noeud* n0,const Noeud* n1,
			  const Noeud* n2,const Noeud* n3);
};




//---------------------------------------------------------------------------------
//----------------------------------Node-------------------------------------------
//---------------------------------------------------------------------------------

void Noeud::UpdateVisibilite(const GReal& imprecision)
{		
GReal distanceAuPlan=
SharedData::m_cameraVolumeVision.Distance(m_position.x,m_position.y,m_position.z);

// TODO-> voir l'imprecision ; doit etre mieux gerer
if (distanceAuPlan<(imprecision+m_heightGrass))
	{
	//forte probabilite de visibilite
	m_isVisible=true;
	m_mustBeUpdated=true;
	}
else if (distanceAuPlan<(imprecision*2.0f+m_heightGrass))
	{
	m_isVisible=false;
	m_mustBeUpdated=true;
	}
else
	{
	m_isVisible=false;
	m_mustBeUpdated=false;
	}
}	






void Noeud::UpdateTypeAffichage()
{	
m_distanceToCamera=sqrt((SharedData::m_cameraPosition.x-m_position.x)
		       *(SharedData::m_cameraPosition.x-m_position.x)
		       +(SharedData::m_cameraPosition.y-m_position.y)
		       *(SharedData::m_cameraPosition.y-m_position.y));
if (m_distanceToCamera<SharedData::m_distance3D)
	{
	m_typeOfDrawing=DRAWGRASS_3D ;
	//transition 3D<->2D
	if (SharedData::m_distance3D>SharedData::m_distance2D5)
	   {
	   GReal rapportDistance=m_distanceToCamera/SharedData::m_distance3D; 
 	   m_heightGrassDraw = (1.0f-rapportDistance)*m_heightGrass +
  		     	       rapportDistance*m_heightGrass*0.1f;
	   }
	else
	   m_heightGrassDraw = m_heightGrass;
	}
else if (m_distanceToCamera<SharedData::m_distance2D5)
	{
	m_typeOfDrawing=DRAWGRASS_2D5 ;
	//transition 2D5<->2D
	GReal rapportDistance =(m_distanceToCamera-SharedData::m_distance3D)/
	 		       (SharedData::m_distance2D5-SharedData::m_distance3D); 
 	m_heightGrassDraw = (1.0f-rapportDistance)*m_heightGrass +
  		     	    rapportDistance*m_heightGrass*0.1f;
	}
else if (m_distanceToCamera<SharedData::m_distance2D)
	{
	m_typeOfDrawing=DRAWGRASS_2D ;
	}
else
	{
	m_typeOfDrawing=DRAWGRASS_NONE;
	}
	
}
void Noeud::UpdateDensite()
{
//if (!calculDensite)
//	{
	m_grassDensityDraw=m_grassDensity;
	return;
//	}
/*Vec3 vecteurVersMarcheur;
vecteurVersMarcheur.x = m_position.x - positionMarcheur.x;
vecteurVersMarcheur.y = m_position.y - positionMarcheur.y;
vecteurVersMarcheur.z = m_position.z - positionMarcheur.z;
if (vecteurVersMarcheur.z==0.0f) vecteurVersMarcheur.z=-0.1f;
vecteurVersMarcheur.normalize();

GReal resultat  = (-(vecteurVersMarcheur*m_groundNormal)+1.0f)*0.5f;
//ici fonction peut etre a ameliorer
//on pourrait desactiver ce calcul pour les noeuds proches
m_grassDensityDraw = 1.0f - (1.0f-resultat)*(1.0f-resultat);
m_grassDensityDraw*=m_grassDensity;*/
}

void Noeud::UpdateInfluencePrimitives()
{
float tmp[3];
grassGetLocalWind(m_position.x, m_position.y,0.0f, 0.0f, 0.0f,tmp[0],tmp[1], tmp[2]);
m_windInformation._power=tmp[0];
m_windInformation._vecteur.x=tmp[1];
m_windInformation._vecteur.y=tmp[2];
}

void Noeud::FabricationHerbe()
{
m_grass.MakeWithWind(m_windInformation,m_heightGrassDraw);
}




void Noeud::UpdatePreDraw()
{
//UpdateTypeAffichage
m_distanceToCamera=sqrt((SharedData::m_cameraPosition.x-m_position.x)
		       *(SharedData::m_cameraPosition.x-m_position.x)
		       +(SharedData::m_cameraPosition.y-m_position.y)
		       *(SharedData::m_cameraPosition.y-m_position.y));
if (m_distanceToCamera<SharedData::m_distance3D)
	{
	m_typeOfDrawing=DRAWGRASS_3D ;
	//transition 3D<->2D
	//if (SharedData::m_distance3D>SharedData::m_distance2D5)
	//   {
	//   GReal rapportDistance=m_distanceToCamera/SharedData::m_distance3D; 
 	//   m_heightGrassDraw = (1.0f-rapportDistance)*m_heightGrass +
  	//	     	       rapportDistance*m_heightGrass*0.1f;
	//   }
	//else
	m_heightGrassDraw = m_heightGrass;
	}
else if (m_distanceToCamera<SharedData::m_distance2D5)
	{
	m_typeOfDrawing=DRAWGRASS_2D5 ;
	//transition 2D5<->2D
	//GReal rapportDistance =(m_distanceToCamera-SharedData::m_distance3D)/
	// 		       (SharedData::m_distance2D5-SharedData::m_distance3D); 
 	//m_heightGrassDraw = (1.0f-rapportDistance)*m_heightGrass +
  	//	     	    rapportDistance*m_heightGrass*0.1f;
	m_heightGrassDraw = m_heightGrass;
	}
else if (m_distanceToCamera<SharedData::m_distance2D)
	{
	m_typeOfDrawing=DRAWGRASS_2D ;
	}
else
	{
	m_typeOfDrawing=DRAWGRASS_NONE;
	return;
	}
//UpdateDensite(...)
m_grassDensityDraw=m_grassDensity;
//UpdateInfluencePrimitives()
float tmp[3];
grassGetLocalWind(m_position.x, m_position.y,0.0f, 0.0f, 0.0f,tmp[0],tmp[1], tmp[2]);
m_windInformation._power=tmp[0];
m_windInformation._vecteur.x=tmp[1];
m_windInformation._vecteur.y=tmp[2];

m_grass.MakeWithWind(m_windInformation,m_heightGrassDraw);
}








//---------------------------------------------------------------------------------
//-------------------------------NodeCrushing-----------------------------------------
//---------------------------------------------------------------------------------

void NodeCrushing::UpdateCrushing(const GReal& dt)
		{
		if (!m_isActive) return;
		if (!m_isCrushed) return;
		
		m_crushingCurrentTime-=dt;
		if (m_crushingCurrentTime<0.0f) {m_crushingCurrentTime=0.0f;
			 			 m_isCrushed=false;} 
		m_crushingCurrentPower=m_crushingMaxPower*
		  pow(m_crushingCurrentTime/m_crushingMaxTime,ms_crushingDamping);	   	
		}


GReal NodeCrushing::UpdateInfluenceTrajectoire(const Point3& debutSegmentTrajectoire,
       					const Point3& finSegmentTrajectoire,
					const Vec2& vDirectionTrajectoire,
					const Vec2& vNormalTrajectoire,
					const GReal& powerCrushing,
					const GReal& coefCrushingAccordingToNormal)
{

//calcul de la distance du noeud a la droite...
GReal distance=m_position.dist2right(debutSegmentTrajectoire,finSegmentTrajectoire);			
GReal coef1=coefCrushingAccordingToNormal*distance;
if (coef1>1.0f) coef1=1.0f;

//calcul du potentiel
GReal potentiel;
if (distance==0.0f)
	{
	//cas particulier et rare
	//ne marche pas alors on modifie un poil la position
	Point3 positionMod = m_position;
	positionMod[1]+=0.01f;
	potentiel = PotentielPoint(positionMod,
				 debutSegmentTrajectoire,
				 finSegmentTrajectoire);
	
	}
else
	potentiel = PotentielPoint(m_position,
				 debutSegmentTrajectoire,
				 finSegmentTrajectoire);	

//si le noeud n'est pas actif pas besoin de continuer
if (!m_isActive) return potentiel;

//activation du noeud
m_isCrushed=true;

//TODO -> a faire plus juste...
//permet de ne pas pencher l'herbe 
//dans l'autre sens si il est deja fortement ecrase...
if (m_crushingCurrentPower>100.0f)
	{
	GReal scalaire2=
	vDirectionTrajectoire*Vec2(m_crushingDirection.x,m_crushingDirection.y);
	if (scalaire2<0.0f)
	 	{
		//--------------------------------puissance
		m_crushingCurrentPower += potentiel * powerCrushing;
		m_crushingMaxPower      = m_crushingCurrentPower;
		//------------------------------------temps
		m_crushingCurrentTime   =  (m_crushingCurrentPower*0.0001f+0.75f)*ms_crushingMaxTime;
		m_crushingMaxTime 	= m_crushingCurrentTime; 
		return potentiel;
		}
	}

//on met le vecteur d'ecrasement a jour...
//pas necessaire avant car seul sa direction compte pour l'affichage
GReal facteur;
if (m_crushingMaxPower==0.0f) facteur=0.0f;	
else facteur = m_crushingCurrentPower/m_crushingMaxPower;
m_crushingDirection.x*=facteur ;
m_crushingDirection.y*=facteur ;
						
//mise a jour des informations d'ecrasement
//--------------------------------direction
m_crushingDirection.x += (1.0f-coef1)*potentiel*vDirectionTrajectoire.x;
m_crushingDirection.y += (1.0f-coef1)*potentiel*vDirectionTrajectoire.y;
Vec2 vecteurVersPosition(m_position.x-debutSegmentTrajectoire.x, 
			 m_position.y-debutSegmentTrajectoire.y);
GReal scalaire = vecteurVersPosition*vNormalTrajectoire;
m_crushingDirection.x += coef1*potentiel*vNormalTrajectoire.x*Signe(scalaire);
m_crushingDirection.y += coef1*potentiel*vNormalTrajectoire.y*Signe(scalaire);
//--------------------------------puissance
m_crushingCurrentPower += potentiel * powerCrushing;
m_crushingMaxPower      = m_crushingCurrentPower;
//------------------------------------temps
m_crushingCurrentTime   =  (m_crushingCurrentPower*0.0001f+0.75f)*ms_crushingMaxTime;
m_crushingMaxTime 	= m_crushingCurrentTime; 

return potentiel;
}

void NodeCrushing::Make(const Noeud* n)
{
m_grass=n->m_grass;
m_grass.ModifyWithCrushing(n->m_windInformation._power,
			   m_crushingDirection, m_crushingCurrentPower,
			   n->m_heightGrass);
}

void NodeCrushing::Make(const Noeud* n0,const Noeud* n1)
{
GReal heightGrass=n0->m_heightGrass+n1->m_heightGrass;
heightGrass*=0.5f;
GReal windPower=n0->m_windInformation._power+n1->m_windInformation._power;
windPower*=0.5f;
m_grass.Make(n0->m_grass,n1->m_grass,0.5f);
m_grass.ModifyWithCrushing(windPower,
			   m_crushingDirection, m_crushingCurrentPower,
			   heightGrass);
}

void NodeCrushing::Make(const Noeud* n0,const Noeud* n1,
			const Noeud* n2,const Noeud* n3)
{
GReal heightGrass=n0->m_heightGrass+n1->m_heightGrass+n2->m_heightGrass+n3->m_heightGrass;
heightGrass*=0.25f;
GReal windPower=n0->m_windInformation._power+n1->m_windInformation._power
	       +n2->m_windInformation._power+n3->m_windInformation._power;
windPower*=0.25f;
m_grass.Make(n0->m_grass,n1->m_grass,n2->m_grass,n3->m_grass,0.5f,0.5f);
m_grass.ModifyWithCrushing(windPower,
			   m_crushingDirection, m_crushingCurrentPower,
			   heightGrass);
}







#endif /* _noeud_ */
